@@ -12,9 +12,7 @@ use File::Spec     ();
 use constant NO_CACHE => $ENV{MOJO_ASSETPACK_NO_CACHE} || 0;
 use constant DEBUG    => $ENV{MOJO_ASSETPACK_DEBUG}    || 0;
 
-our $VERSION = '0.51';
-
-my $MTIME = time;
+our $VERSION = '0.52';
 
 has base_url      => '/packed/';
 has minify        => 0;
@@ -37,7 +35,7 @@ sub add {
 
 sub fetch {
   my $self = shift;
-  $self->_fetch(@_, $self->out_dir)->url;
+  $self->_handler('https')->asset_for(shift, $self)->in_memory(!$self->out_dir)->save->path;
 }
 
 sub get {
@@ -106,7 +104,9 @@ sub register {
 
 sub _asset {
   my ($self, $name) = @_;
-  $self->{asset}{$name} ||= Mojolicious::Plugin::AssetPack::Asset->new(url => $name);
+  my $asset = $self->{asset}{$name} ||= Mojolicious::Plugin::AssetPack::Asset->new;
+  $asset->path(File::Spec->catfile($self->out_dir, $name)) unless $asset->path;
+  $asset;
 }
 
 sub _assets {
@@ -127,8 +127,8 @@ sub _assets_from_memory {
       return unless $path->[1] and 0 == index "$path", $self->base_url;
       return unless my $asset = $c->asset->_asset($path->[1]);
       return if $asset->{internal};
-      $c->res->headers->last_modified(Mojo::Date->new($MTIME))
-        ->content_type($c->app->types->type($asset->url =~ /\.(\w+)$/ ? $1 : 'txt') || 'text/plain');
+      $c->res->headers->last_modified(Mojo::Date->new($^T))
+        ->content_type($c->app->types->type($asset->path =~ /\.(\w+)$/ ? $1 : 'txt') || 'text/plain');
       $c->reply->asset($asset);
     }
   );
@@ -154,32 +154,6 @@ sub _build_out_dir {
   return $out_dir // '';
 }
 
-sub _fetch {
-  my ($self, $url, $save) = @_;
-  my $lookup = _name($url);
-  my $asset;
-
-  if ($asset = $self->_find('packed', qr{^$lookup\.\w+$})) {
-    $self->_app->log->debug("Asset $url is already downloaded") if DEBUG;
-    return $asset;
-  }
-
-  my $res = $self->_ua->get($url)->res;
-  my $ct  = $res->headers->content_type // 'text/plain';
-  my $ext = Mojolicious::Types->new->detect($ct) || 'txt';
-
-  if (my $e = $res->error) {
-    die "Asset $url could not be fetched: $e->{message}";
-  }
-
-  $ext = $ext->[0] if ref $ext;
-  $ext = Mojo::URL->new($url)->path =~ m!\.(\w+)$! ? $1 : 'txt' if !$ext or $ext eq 'bin';
-  $self->_app->log->info("Asset $url was fetched");
-  $asset = $self->_asset("$lookup.$ext")->url($url)->add_chunk($res->body);
-  $asset->in_memory(0)->url(File::Spec->catfile($self->out_dir, "$lookup.$ext"))->save if $save;
-  $asset;
-}
-
 sub _find {
   my $needle = pop;
   my $self   = shift;
@@ -191,13 +165,22 @@ sub _find {
   for my $path (map { File::Spec->catdir($_, @path) } @{$self->_app->static->paths}) {
     opendir my $DH, $path or next;
     for (readdir $DH) {
-      /$needle/ and return $self->_asset($_)->url(Cwd::abs_path(File::Spec->catfile($path, $_)))->in_memory(0);
+      /$needle/ and return $self->_asset($_)->path(Cwd::abs_path(File::Spec->catfile($path, $_)))->in_memory(0);
     }
   }
 
   return undef;
 }
 
+sub _handler {
+  my ($self, $moniker) = @_;
+  $self->{handler}{$moniker} ||= do {
+    my $class = "Mojolicious::Plugin::AssetPack::Handler::" . ucfirst $moniker;
+    eval "require $class;1" or die "Could not load $class: $@\n";
+    $class->new;
+  };
+}
+
 sub _inject {
   my ($self, $c, $moniker, $args, @attrs) = @_;
   my $tag_helper = $moniker =~ /\.js/ ? 'javascript' : 'stylesheet';
@@ -248,8 +231,8 @@ sub _process {
   my ($asset, $file, $re, @checksum);
 
   @sources = map {
-    my $asset = $self->_find(split '/', $_) || (/^https?:/ ? $self->_fetch($_, $self->out_dir) : $self->_fetch($_, ''));
-    push @checksum, $self->preprocessors->checksum(_ext($_), \$asset->slurp, $asset->url);
+    my $asset = $self->_source_for_url($_);
+    push @checksum, $self->preprocessors->checksum(_ext($_), \$asset->slurp, $asset->path);
     $asset;
   } @sources;
 
@@ -263,24 +246,24 @@ sub _process {
   }
 
   $asset = $self->{asset}{$file} = Mojolicious::Plugin::AssetPack::Asset->new;
-  $asset->in_memory(1)->url(File::Spec->catfile($self->out_dir, $file));
+  $asset->in_memory(1)->path(File::Spec->catfile($self->out_dir, $file));
 
   for my $source (@sources) {
     eval {
       my $content = $source->slurp;
-      $self->preprocessors->process(_ext($source->url), $self, \$content, $source->url);
-      $asset->add_chunk($content);
+      $self->preprocessors->process(_ext($source->path), $self, \$content, $source->path);
+      $asset->content($asset->content . $content);
       1;
     } or do {
       my $e = $@;
-      warn "[ASSETPACK] process(@{[$source->url]}) FAIL $e\n" if DEBUG;
-      $asset->url(File::Spec->catfile($self->out_dir, "$name-$checksum[0].err.$ext"));
-      $asset->add_chunk($self->_make_error_asset($moniker, $source->basename, $e || 'Unknown error'));
+      warn "[ASSETPACK] process(@{[$source->path]}) FAIL $e\n" if DEBUG;
+      $asset->path(File::Spec->catfile($self->out_dir, "$name-$checksum[0].err.$ext"));
+      $asset->content($self->_make_error_asset($moniker, $source->basename, $e || 'Unknown error'));
       last;
     };
   }
 
-  $asset->in_memory($self->out_dir ? 0 : 1)->save;
+  $asset->in_memory(!$self->out_dir)->save;
   $self->_app->log->info("Built asset for $moniker");
   $asset;
 }
@@ -298,7 +281,7 @@ sub _reloader {
   return if !$config->{enabled} and $app->mode ne 'development';
 
   warn "[ASSETPACK] Adding reloader asset and route\n" if DEBUG;
-  $reloader->url('reloader.js')->{internal} = 1;
+  $reloader->path('reloader.js')->{internal} = 1;
   $self->{assets}{'reloader.js'} = [$reloader];
   push @{$app->renderer->classes}, __PACKAGE__;
   $app->routes->get('/packed/reloader')->to(template => 'packed/reloader', strategy => 'document', %$config);
@@ -309,6 +292,28 @@ sub _reloader {
   )->name('assetpack.ws');
 }
 
+sub _source_for_url {
+  my $self = shift;
+  my $url  = Mojo::URL->new(shift);
+  my $asset;
+
+  if (my $scheme = $url->scheme) {
+    my $lookup = _name($url);
+
+    if ($asset = $self->_find('packed', qr{^$lookup\.\w+$})) {
+      $self->_app->log->debug("Asset $url is fetched") if DEBUG;
+    }
+    else {
+      $asset = $self->_handler($scheme)->asset_for($url, $self)->in_memory(!$self->out_dir)->save;
+    }
+  }
+  else {
+    $asset = $self->_find(split '/', $url) || $self->_handler('https')->asset_for($url, $self);
+  }
+
+  return $asset;
+}
+
 # utils
 sub _ext { local $_ = File::Basename::basename($_[0]); /\.(\w+)$/ ? $1 : 'unknown'; }
 
@@ -329,7 +334,7 @@ Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass, javascrip
 
 =head1 VERSION
 
-0.51
+0.52
 
 =head1 SYNOPSIS
 
