@@ -21,77 +21,76 @@ use Mojo::Util     ();
 use Cwd            ();
 use Carp           ();
 use File::Basename ();
+use constant DEBUG => $ENV{MOJO_ASSETPACK_DEBUG} || 0;
 
 =head1 ATTRIBUTES
 
+=head2 content
+
+  $self = $self->content($data);
+  $data = $self->content;
+
 =head2 in_memory
 
 Boolean true if this file only exists in memory, false if stored on disk.
 
-=head2 url
+=head2 path
 
-Either location on disk, HTTP URL or a virtual location (in case of L</in_memory>).
+Either location on disk or a virtual location.
 
 =cut
 
+has content   => '';
 has in_memory => 1;
-has url       => '';
+has path      => '';
 
 =head1 METHODS
 
-=head2 add_chunk
-
-  $self = $self->add_chunk($bytes);
-
-Will store data internally, which later can be written to disk using L</save>.
-
-=cut
-
-sub add_chunk {
-  my ($self, $chunk) = @_;
-  $self->{content} //= '';
-  $self->{content} .= $chunk;
-  $self;
-}
-
 =head2 basename
 
-Returns the basename of L</url>.
+Returns the basename of L</path>.
 
 =cut
 
-sub basename { File::Basename::basename(shift->url); }
+sub basename { File::Basename::basename(shift->path); }
 
 =head2 slurp
 
   $bytes = $self->slurp;
 
-Read in the contents of the asset. Returns the data from L</add_chunk>
+Read in the contents of the asset. Returns the data from L</content>
 if L</in_memory> is true.
 
 =cut
 
 sub slurp {
   my $self = shift;
-  $self->in_memory ? $self->{content} // '' : Mojo::Util::slurp($self->url);
+  $self->in_memory ? $self->{content} // '' : Mojo::Util::slurp($self->path);
 }
 
 =head2 save
 
   $self = $self->->save;
 
-L</save> is used to write all the L</add_chunk> data to disk.
-
-This method does nothing if L</in_memory> is true.
+L</save> is used to write L</content> to disk. This method does nothing if
+L</in_memory> is true.
 
 =cut
 
 sub save {
   my $self = shift;
 
-  return $self if $self->in_memory;
-  die "Nothing to save to @{[$self->url]}" unless defined $self->{content};
-  Mojo::Util::spurt(delete $self->{content}, $self->url);
+  if (not defined $self->{content}) {
+    die "Cannot save empty asset to save to @{[$self->path]}";
+  }
+  elsif ($self->in_memory) {
+    warn "[ASSETPACK] Skip save of @{[$self->path]}\n" if DEBUG;
+  }
+  else {
+    warn "[ASSETPACK] Save @{[$self->path]}\n" if DEBUG;
+    Mojo::Util::spurt(delete $self->{content}, $self->path);
+  }
+
   return $self;
 }
 
